;*******************************************************************************
;*
;*	Titel:	Unterprogramm zum Empfang eines DCF77-Signal (SR_DCF77.asm)
;*
;*	Datum:	15/08/09	Version:	0.1
;*	Autor:	WEIGU
;*
;*	Informationen zur Beschaltung:
;*	Prozessor:	ATmega            Quarzfrequenz:
;*	Eingaenge:      Data -> INT1 (Pin 5 (PD3) beim ATmega8)
;*	Ausgaenge:	PON (Pin 4 (PD2) beim ATmega8)
;*
;*      !! DCF1-Empfangsmodul von Pollin !!
;*
;*	Informationen zur Funktionsweise:
;*      Nutzt externen Interrupt 1
;*      


;*	
;*******************************************************************************

;-------------------------------------------------------------------------------
;	Externen Interrupt INT1 und Empfaenger einschalten
;-------------------------------------------------------------------------------
DCFON:  push    Tmp1             ;benutzte Reg. retten (Tmp1 = Zwischenspeicher)
        clr     Tmp1
        clr     MinDCF
        clr     HrsDCF
        sts     SECPNT,Tmp1
        sts     FR_END,Tmp1
        sts     FR_PNT,Tmp1
        sts     FINFNR,Tmp1
        sts     TSTFLG,Tmp1
        sts     DCFERR,Tmp1
        sts     DCFSTS,Tmp1
        in      Tmp1,MCUCR      ;MCU Control Register einlesen
        andi    Tmp1,0b11110111 ;
        ori     Tmp1,0b00000100 ;ISC11=0, ISC10=1 => jede Pegelaenderung
        out     MCUCR,Tmp1      ;Wert ins MCUCR zurückschreiben
        in      Tmp1,GICR       ;General Int. Control Reg. einlesen
        ori     Tmp1,0x80       ;INT1=1    INT aktivieren
        out     GICR,Tmp1       ;GICR setzen
        sbi     PORTD,2         ;PON auf 5V
        rcall   W100ms
        cbi     PORTD,2         ;PON auf 0V (Flanke schaltet Empf. ein)
        rcall   W100ms
        sei                     ;IRQs global zulassen
        pop     Tmp1            ;Werte der geretteten Register wiederherstellen
        ret

;-------------------------------------------------------------------------------
;       DCF77-Empfaenger und Interrupts (INT0 TOIE2) deaktivieren
;-------------------------------------------------------------------------------
DCFOFF: push    Tmp1             ;benutzte Reg. retten (Tmp1 = Zwischenspeicher)
        in      Tmp1,GICR       ;General Int. Control Reg. einlesen
        andi    Tmp1,0x7F       ;INT1=0    INT deaktivieren
        out     GICR,Tmp1       ;GICR setzen
        in      Tmp1,TIMSK       ;steigende Flanke: stoppe den Timer
        andi    Tmp1,0xBF        ;sperre Timer2 Ueberlauf-Interrupt
        out     TIMSK,Tmp1
        clr     Tmp1             ;Timer ausschalten
        out     TCCR2,Tmp1
        sbi     PORTD,2         ;PON auf 5V Empfaenger aus
        cli                     ;IRQs global sperren
        sbi     PORTD,1         ;LED EIN TesT11
        pop     Tmp1            ;Werte der geretteten Register wiederherstellen
        ret

;-------------------------------------------------------------------------------
;       Empfangene Informationen dekodieren
;-------------------------------------------------------------------------------
DCODE:  push    Tmp1            ;benutzte Reg. retten (Tmp1 = Zwischenspeicher)
        lds     Tmp1,C_10ms
        cpi     Tmp1,175        ;ueberpruefe ob 59. Minute
        brlo    DCODE1
        lds     Tmp1,C_10ms
        cpi     Tmp1,195
        brsh    DCODE1
        ldi     Tmp1,255        ;falls ja Sekundenzeiger auf 0 (255 wegen incr.)
        sts     SECPNT,Tmp1
        clr     Tmp1
        sts     FR_END,Tmp1     ;Ende des Rahmens: 0 in der 59 Sek. (1 in 57s)
        lds     Tmp1,FR_PNT
        inc     Tmp1            ;erhoehe den Rahmenzeiger
        sts     FR_PNT,Tmp1
        rjmp    DCODER          ;Raus
DCODE1: lds     Tmp1,C_10ms
        cpi     Tmp1,87         ;ueberpruefe ob Bit = 0 (900ms = 1s-100ms)
        brlo    DCODE2
        lds     Tmp1,C_10ms
        cpi     Tmp1,95
        brsh    DCODE2
        lds     Tmp1,SECPNT
        inc     Tmp1            ;erhoehe Sekundenzeiger
        sts     SECPNT,Tmp1
        clr     Tmp1
        sts     DCFBIT,Tmp1     ;Bit = 0
        rjmp    DCODE5
DCODE2: lds     Tmp1,C_10ms
        cpi     Tmp1,77         ;ueberpruefe ob Bit = 1 (800ms = 1s-200ms)
        brlo    DCODE3
        lds     Tmp1,C_10ms
        cpi     Tmp1,85
        brsh    DCODE3
        lds     Tmp1,SECPNT
        inc     Tmp1            ;erhoehe Sekundenzeiger
        sts     SECPNT,Tmp1
        ldi     Tmp1,1          ;Bit = 1
        sts     DCFBIT,Tmp1
        rjmp    DCODE5
DCODE3: ldi     Tmp1,99         ;Sekundenzeiger = 99 (Fehler)
        sts     SECPNT,Tmp1
        lds     Tmp1,DCFERR
        inc     Tmp1            ;erhoehe Zaehler fuer fehlerhaft empfangene Bits
        sts     DCFERR,Tmp1
        rjmp    DCODER          ;Raus
DCODE5: lds     Tmp1,SECPNT     ;Sekunde Nr 21?
        cpi     Tmp1,21
        brne    DCD22
        lds     Tmp1,DCFBIT     ;falls ja, Bit = 1?
        cpi     Tmp1,1
        brne    DCD210
        ori     MinDCF,0x01     ;falls Bit = 1, dann Bit in Minutenreg. setzen
        rjmp    DCODER          ;Raus
DCD210: andi    MinDCF,0xFE     ;falls Bit = 0 ,dann Bit in Minutenreg. loeschen
        rjmp    DCODER          ;Raus
DCD22:  lds     Tmp1,SECPNT     ;Sekunde Nr 22?
        cpi     Tmp1,22
        brne    DCD23
        lds     Tmp1,DCFBIT     ;falls ja, Bit = 1?
        cpi     Tmp1,1
        brne    DCD220
        ori     MinDCF,0x02     ;falls Bit = 1, dann Bit in Minutenreg. setzen
        rjmp    DCODER          ;Raus
DCD220: andi    MinDCF,0xFD     ;falls Bit = 0, dann Bit in Minutenreg. loeschen
        rjmp    DCODER          ;Raus
DCD23:  lds     Tmp1,SECPNT     ;Sekunde Nr 23?
        cpi     Tmp1,23
        brne    DCD24
        lds     Tmp1,DCFBIT     ;falls ja, Bit = 1?
        cpi     Tmp1,1
        brne    DCD230
        ori     MinDCF,0x04     ;falls Bit = 1, dann Bit in Minutenreg. setzen
        rjmp    DCODER          ;Raus
DCD230: andi    MinDCF,0xFB     ;falls Bit = 0 ,dann Bit in Minutenreg. loeschen
        rjmp    DCODER          ;Raus
DCD24:  lds     Tmp1,SECPNT     ;Sekunde Nr 24?
        cpi     Tmp1,24
        brne    DCD25
        lds     Tmp1,DCFBIT     ;falls ja, Bit = 1?
        cpi     Tmp1,1
        brne    DCD240
        ori     MinDCF,0x08     ;falls Bit = 1, dann Bit in Minutenreg. setzen
        rjmp    DCODER          ;Raus
DCD240: andi    MinDCF,0xF7     ;falls Bit = 0 ,dann Bit in Minutenreg. loeschen
        rjmp    DCODER          ;Raus
DCD25:  lds     Tmp1,SECPNT     ;Sekunde Nr 25?
        cpi     Tmp1,25
        brne    DCD26
        lds     Tmp1,DCFBIT     ;falls ja, Bit = 1?
        cpi     Tmp1,1
        brne    DCD250
        ori     MinDCF,0x10     ;falls Bit = 1, dann Bit in Minutenreg. setzen
        rjmp    DCODER          ;Raus
DCD250: andi    MinDCF,0xEF     ;falls Bit = 0 ,dann Bit in Minutenreg. loeschen
        rjmp    DCODER          ;Raus
DCD26:  lds     Tmp1,SECPNT     ;Sekunde Nr 26?
        cpi     Tmp1,26
        brne    DCD27
        lds     Tmp1,DCFBIT     ;falls ja, Bit = 1?
        cpi     Tmp1,1
        brne    DCD260
        ori     MinDCF,0x20     ;falls Bit = 1, dann Bit in Minutenreg. setzen
        rjmp    DCODER          ;Raus
DCD260: andi    MinDCF,0xDF     ;falls Bit = 0 ,dann Bit in Minutenreg. loeschen
        rjmp    DCODER          ;Raus
DCD27:  lds     Tmp1,SECPNT     ;Sekunde Nr 27?
        cpi     Tmp1,27
        brne    DCD29
        lds     Tmp1,DCFBIT     ;falls ja, Bit = 1?
        cpi     Tmp1,1
        brne    DCD270
        ori     MinDCF,0x40     ;falls Bit = 1, dann Bit in Minutenreg. setzen
        rjmp    DCODER          ;Raus
DCD270: andi    MinDCF,0xBF     ;falls Bit = 0 ,dann Bit in Minutenreg. loeschen
        rjmp    DCODER          ;Raus
DCD29:  lds     Tmp1,SECPNT     ;Sekunde Nr 29?
        cpi     Tmp1,29
        brne    DCD30
        lds     Tmp1,DCFBIT     ;falls ja, Bit = 1?
        cpi     Tmp1,1
        brne    DCD290
        ori     HrsDCF,0x01     ;falls Bit = 1, dann Bit in Stundenreg. setzen
        rjmp    DCODER          ;Raus
DCD290: andi    HrsDCF,0xFE     ;falls Bit = 0 ,dann Bit in Stundenreg. loeschen
        rjmp    DCODER          ;Raus
DCD30:  lds     Tmp1,SECPNT     ;Sekunde Nr 30?
        cpi     Tmp1,30
        brne    DCD31
        lds     Tmp1,DCFBIT     ;falls ja, Bit = 1?
        cpi     Tmp1,1
        brne    DCD300
        ori     HrsDCF,0x02     ;falls Bit = 1, dann Bit in Stundenreg. setzen
        rjmp    DCODER          ;Raus
DCD300: andi    HrsDCF,0xFD     ;falls Bit = 0 ,dann Bit in Stundenreg. loeschen
        rjmp    DCODER          ;Raus
DCD31:  lds     Tmp1,SECPNT     ;Sekunde Nr 31?
        cpi     Tmp1,31
        brne    DCD32
        lds     Tmp1,DCFBIT     ;falls ja, Bit = 1?
        cpi     Tmp1,1
        brne    DCD310
        ori     HrsDCF,0x04     ;falls Bit = 1, dann Bit in Stundenreg. setzen
        rjmp    DCODER          ;Raus
DCD310: andi    HrsDCF,0xFB     ;falls Bit = 0 ,dann Bit in Stundenreg. loeschen
        rjmp    DCODER          ;Raus
DCD32:  lds     Tmp1,SECPNT     ;Sekunde Nr 32?
        cpi     Tmp1,32
        brne    DCD33
        lds     Tmp1,DCFBIT     ;falls ja, Bit = 1?
        cpi     Tmp1,1
        brne    DCD320
        ori     HrsDCF,0x08     ;falls Bit = 1, dann Bit in Stundenreg. setzen
        rjmp    DCODER          ;Raus
DCD320: andi    HrsDCF,0xF7     ;falls Bit = 0 ,dann Bit in Stundenreg. loeschen
        rjmp    DCODER          ;Raus
DCD33:  lds     Tmp1,SECPNT     ;Sekunde Nr 33?
        cpi     Tmp1,33
        brne    DCD34
        lds     Tmp1,DCFBIT     ;falls ja, Bit = 1?
        cpi     Tmp1,1
        brne    DCD330
        ori     HrsDCF,0x10     ;falls Bit = 1, dann Bit in Stundenreg. setzen
        rjmp    DCODER          ;Raus
DCD330: andi    HrsDCF,0xEF     ;falls Bit = 0 ,dann Bit in Stundenreg. loeschen
        rjmp    DCODER          ;Raus
DCD34:  lds     Tmp1,SECPNT     ;Sekunde Nr 34?
        cpi     Tmp1,34
        brne    DCD57
        lds     Tmp1,DCFBIT     ;falls ja, Bit = 1?
        cpi     Tmp1,1
        brne    DCD340
        ori     HrsDCF,0x20     ;falls Bit = 1, dann Bit in Stundenreg. setzen
        rjmp    DCODER          ;Raus
DCD340: andi    HrsDCF,0xDF     ;falls Bit = 0 ,dann Bit in Stundenreg. loeschen
        rjmp    DCODER          ;Raus
DCD57:  lds     Tmp1,SECPNT     ;Sekunde Nr 34?
        cpi     Tmp1,57
        brne    DCODER
        ldi     Tmp1,1          ;Rahmen fertig (1)
        sts     FR_END,Tmp1
        lds     Tmp1,FINFNR     ;Erhoehe Zaehler der fertig empfangenen Rahmen
        inc     Tmp1
        sts     FINFNR,Tmp1
DCODER: pop     Tmp1            ;Werte der geretteten Register wiederherstellen
        ret                     ;Rücksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Zeitsignal auf Konsistenz ueberpruefen
;-------------------------------------------------------------------------------
        ;DCFSTS (Status) = 0  noch nicht fertig oder fehlerhafter Rahmen
        ;                  1  Erfolg, aber erst 1*: noch nicht fertig!
        ;                  2  Erfolg, aber erst 2*: noch nicht fertig!
        ;                  3  Erfolg! 3* richtiger Wert!
        ;                  4  55* probiert schlechter Empfang!
        ;                  5  Fehler nach erstem erfolgreichem Rahmen
        ;Das Signal muss dreimal hintereinander fehlerfrei empfangen werden, 
        ;damit es freigegeben wird (3).
        ;Die Variable TSTFLG dient der Ueberpruefung dieser Bedingung.
        ;Sie wird beim ersten Mal auf die Rahmennummer gesetzt (FINFNR). Zwei
        ;weitere Rhamen muessen mit laufender Nummer richtig empfangen werden.

TSTRS:  push    Tmp1            ;benutzte Reg. retten (Tmp1 = Zwischenspeicher)
        push    Tmp2
        lds     Tmp1,FR_END     ;Falls kein Rahmenende, Fehler Status = 0
        cpi     Tmp1,0
        brne    TSTRS6
        rjmp    TSTRS0
TSTRS6: lds     Tmp1,TSTFLG     ;Falls TSTFLG = 0, dann kopiere Min. und Stunden
        cpi     Tmp1,0
        brne    TSTRS7    
        sts     MiBuf1,MinDCF
        sts     HrBuf1,HrsDCF
        lds     Tmp1,FINFNR     ;TSTFLG = FINFNR
        sts     TSTFLG,Tmp1
        rjmp    TSTRS1          ;Status = 1
TSTRS7: lds     Tmp1,FINFNR     ;TSTFLG+1 == FINFNR? (2 Rahmen!)
        lds     Tmp2,TSTFLG
        inc     Tmp2
        cp      Tmp1,Tmp2
        brne    TSTRS8
        sts     MiBuf2,MinDCF
        sts     HrBuf2,HrsDCF
        rjmp    TSTRS2          ;Status = 2
TSTRS8: lds     Tmp1,FINFNR     ;TSTFLG+2 == FINFNR? (3 Rahmen!)
        lds     Tmp2,TSTFLG
        inc     Tmp2
        inc     Tmp2
        cp      Tmp1,Tmp2
        brne    TSTRS9
        lds     Tmp1,MiBuf1
        inc     Tmp1
        lds     Tmp2,MiBuf2
        cp      Tmp2,Tmp1
        brne    TSTRS9
        lds     Tmp1,MiBuf2
        inc     Tmp1
        cp      MinDCF,Tmp1
        brne    TSTRS9
        lds     Tmp1,HrBuf1
        lds     Tmp2,HrBuf2
        cp      Tmp2,Tmp1
        brne    TSTRS9
        cp      HrsDCF,Tmp2
        brne    TSTRS9
        rcall   DCFOFF
        rjmp    TSTRS3          ;Status = 3 Erfolg!!
TSTRS9: lds     Tmp1,FR_PNT
        cpi     Tmp1,60
        breq    TSTRS4          ;Status = 4 schlechter Empfang
        clr     Tmp1
        sts     TSTFLG,Tmp1
TSTRS5: ldi     Tmp1,5          ;Status = 5
        sts     DCFSTS,Tmp1
        rjmp    TSTRSR
TSTRS4: ldi     Tmp1,4          ;Status = 4
        sts     DCFSTS,Tmp1
        rjmp    TSTRSR
TSTRS3: ldi     Tmp1,3          ;Status = 3
        sts     DCFSTS,Tmp1
        rjmp    TSTRSR
TSTRS2: ldi     Tmp1,2          ;Status = 2
        sts     DCFSTS,Tmp1
        rjmp    TSTRSR
TSTRS1: ldi     Tmp1,1          ;Status = 1
        sts     DCFSTS,Tmp1
        rjmp    TSTRSR
TSTRS0: clr     Tmp1            ;Status = 0
        sts     DCFSTS,Tmp1
TSTRSR: pop     Tmp2            ;Werte der geretteten Register wiederherstellen
        pop     Tmp1
        ret
