;*******************************************************************************
;*
;*	Titel:	Bibliothek mit Zeitunterprogrammen (SR_TIME_8M.asm)
;*	Datum:	10/05/07	Version:	0.2	
;*	Autor:	WEIGU
;*
;*	Informationen zur Beschaltung:
;*	Prozessor:	ATMEGA32	Quarzfrequenz:	 extern 8MHz
;*			
;*	Informationen zur Funktionsweise:
;*
;*	Die Bibliothek enthaelt Zeitschleifenunterprogramme zur Verwendung mit
;*	einem externen 8 MHz Quarz.
;*			
;*******************************************************************************

;-------------------------------------------------------------------------------
;       16-Bit-Zeitschleife wartet 1ms (Wait 1ms)
;-------------------------------------------------------------------------------
W1ms:   push    WL      ;rette verwendete Registerinhalte
        push    WH
        ldi     WL,0xCC ;Initialisiere den Schleifenzaehler W
        ldi     WH,0x07 ;G = (t-tT)/4tT = 1ms/4*125ns-1/4 = 2000-1/4
                        ;Korrektur: -15tT fuer rcall, 2 push, 2 pop, ret
                        ;1 Durchlauf der Schleife benoetigt 4tT
                        ;GKorrektur = G-1/4-15/4 = 2000-4 = 1996 = 0x07CC
W1msL:  sbiw    WL,1    ;Dekrementiere den Schleifenzaehler W
        brne    W1msL   ;Verlasse die Schleife bei Schleifenzaehler W=0
        pop     WH      ;Wiederherstellung der verwendeten Registerinhalte
        pop     WL
        ret             ;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       16-Bit-Zeitschleife wartet 10ms (Wait 10ms)
;-------------------------------------------------------------------------------
W10ms:  push    WL      ;rette verwendete Registerinhalte
        push    WH
        ldi     WL,0x1C ;Initialisiere den Schleifenzaehler W
        ldi     WH,0x4E ;G = (t-tT)/4tT = 10ms/4*125ns-1/4 = 20000-1/4
                        ;Korrektur: -15tT fuer rcall, 2 push, 2 pop, ret
                        ;1 Durchlauf der Schleife benoetigt 4tT
                        ;GKorrektur = G-1/4-15/4 = 20000-4 = 19996 = 0x4E1C
W10msL: sbiw    WL,1    ;Dekrementiere den Schleifenzaehler W
        brne    W10msL  ;Verlasse die Schleife bei Schleifenzaehler W=0
        pop     WH      ;Wiederherstellung der verwendeten Registerinhalte
        pop     WL
        ret             ;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet 100ms (Wait 100ms)
;       Der Fehler durch zusaetz. Befehle kann hier vernachlässigt werden.
;-------------------------------------------------------------------------------

W100ms: push    XL      ;rette verwendete Registerinhalte
        ldi     XL,0x0A ;Zaehler = 10d
W100msL: rcall  W10ms   ;Rufe innere Schleife (Unterprogramm W10ms)
        dec     XL      ;Dekrementiere den Schleifenzaehler X
        brne    W100msL ;Verlasse die Schleife bei Schleifenzaehler XL=0
        pop     XL      ;Wiederherstellung der verwendeten Registerinhalte
        ret             ;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet X mal 1ms (Wait X*1ms)
;	Der Anfangswert von X muss im Hauptprogramm gesetzt werden!
;	Der Fehler durch die zusaetzlichen Befehle kann hier vernachlässigt
;	werden (max Fehler bei einmaligem Aufruf < 0,12%).
;-------------------------------------------------------------------------------
WX1ms:	push	XL	;rette verwendete Registerinhalte
	push	XH
WX1msL:	rcall	W1ms	;Rufe innere Schleife (Unterprogramm W1ms)
	sbiw	XL,1	;Dekrementiere den Schleifenzaehler X
	brne	WX1msL	;Verlasse die Schleife bei Schleifenzaehler X=0
	pop	XH	;Wiederherstellung der verwendeten Registerinhalte
	pop	XL	
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
